# -*- coding: utf-8 -*-
"""

    Тесты инструментов форматирования

"""
# модули проекта
from trivial_tools.formatters import *


# def test_decorate_normal():
#     assert decorate('test') == ''
#
#
# def test_decorate_symbol():
#     assert decorate('test', '%') == ''
#
#
# def test_decorate_line():
#     assert decorate('test', line_width=5) == ''


# def test_decorate_all():
#     assert decorate('test', '$', 4) == ''
